
# This code is a part of Slash, and is released under the GPL.
# Copyright 1997-2001 by Open Source Development Network. See README
# and COPYING for more information, or see http://slashcode.com/.

# This file has seen modifications to turn this file, which was at one point a Perl CGI script into a Apache mod_perl2 module handler. This file and code in it is derived from works of the original authors and copyright owners (Open Source Developers Network). The software package "Slashcode" was released as GPL. This gives others the legal right to use, modify, and redistribute modified versions of Slashcode.

# Code was added to the GPL'ed version of what OSDN has released at version 2.2.6. Namely, code was added so that users client browsers can authenticated using passwords and athentication data stored in HTTP cookies. These modifications are copyright 2014 by Troye "taj" Johnson <troyejohnson@yandex.com>. The license for these modifications is GPLv2, or later.


package slash_index_pl;

use strict;

use Slash;
use Slash::Display;
use Slash::Utility;

use Apache2::Const qw(REDIRECT OK);

# check if $ENV{SLASH_VIRTUAL_USER} and $ENV{SLASH_CONFIG} defined in httpd.conf
slash_preinit_check();

# comes from httpd.conf
my $VIRTUAL_USER = $ENV{SLASH_VIRTUAL_USER};

# must be called before handler() is run
slash_api_init($VIRTUAL_USER);


=item slash_index_pl::handler

Resides in: slash_index_pl.pm

This handler is for the /index.pl URL. Since this is a modperl handler, the '<Location></Location>' tags will have to set the URL, so that Apache will map the URL to the proper module handler in order for the request to be served.

Most of these params will not be handled by the handler() or interpret their meaning. main() will handle that stuff.

All handler does is some pre-processing to set-up the main SLASH_STRUCTS (you can fetch these with getCurrentUser, getCurrentForm, and of course getCurrentDB, most or all of the getCurrent*() subs are used to fetch so call "SLASH_STRUCTS". They are just HASH references used by the API.) so that the handler() can call main(), and thus the module handler has done it's job.

Argument list for this handler. "op" seems like the /index.pl links back to itself so that the site can re-display the page. "section" seems to also refer back to the site to display things.

USER_STRUCT; needed

=cut

sub handler {
    my ( $r ) = @_;

    die "slash_users_pl: slash_api_init() not called"
	unless SLASH_API_INIT_OK;

    my $uid_Param = undef; # This available after authentication process

    my $form_Param = slash_get_params( $r );
    my $cookie_args_Param = slash_get_cookies( $r );
    my $uri_Param = $r->uri(); # e.g, /index.html, /index.pl, or /foo.(html|pl)

    my $method_Param = $r->the_request(); $method_Param =~ s/^(\S*)/$1/;
    $method_Param = $1; # should now be either GET, POST

    my ($unickname, $upasswd); # params from form
    $unickname = $form_Param->{unickname} if $form_Param->{unickname};
    $upasswd = $form_Param->{upasswd} if $form_Param->{upasswd};

    #
    # Attempt to authenticate user
    #

    # $auth_type: 1 is for form login. 2 is for authentication via cookie. Set this to "2" as /users.pl is the only script which can auth w/ "auth_type = 1" -- which is for username and password logins only. This script can authenticate only using cookies.
    my $auth_type = 2;
 
    my $authing_user = undef; # uid of authenticated user, else AnonymousCoward
    my $auth_success = 0; # 1 for successful auth, 0 for authentication failure
    my $auth_uid_field = 0; # First element in array returned by Slash::DB::gutUserAuthenticate()
      
    # For cookie authentication: authenticate with cookie
    if ($auth_type == 2) {
	( $unickname, $upasswd ) = eatUserCookie($cookie_args_Param->{users});

	$authing_user = slash_web_auth( $unickname, $upasswd, 2);
	$auth_success = 1 if $authing_user;

	# `logger 'handler: auth type => 2'`; 
	# `logger 'handler: users cookie in val => '$cookie_args->{users}''`;
	# `logger 'handler: eatUserCookie: n => $unickname, p => $upasswd'`; 
    }
    
    
    if ($auth_success == 1) {
	$uid_Param = $authing_user->[$auth_uid_field];    
    }

    # ** DISABLED: this code designed to follow up after a "auth_type = 1" login
    #
    # if ( ($auth_success == 1) && ($auth_type == 1) ) {
    # 	# ** Set cookie if auth_type is login
	
    # 	# Does not matter whether $upasswd is encrypted or not
    # 	my $users_cookie_data
    # 	    = bakeUserCookie( $authing_user->[$auth_uid_field], $upasswd);
    # 	setCookie('users', $users_cookie_data);
	
    # 	# `logger 'users cookie out value => '$users_cookie_data''`;
    # 	# `logger 'handler: auth was good'`;
    # }
    

    # If User did not authenticate, then they will be anonymous
    if (! defined $uid_Param) {
	$uid_Param = SLASH_ANON_COWARD_UID();
	# `logger 'handler: auth not successful'`; 
    }

    createEnvironment($VIRTUAL_USER, $form_Param, $uid_Param, $uri_Param, $cookie_args_Param, $method_Param);
    main();
    return Apache2::Const::OK;
}


# ** Does most of the work. When Slashcode was on modperlv1, this was the
# index.pl CGI script. Now, it's a module handler.
#
sub main {
    
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();
    my $form      = getCurrentForm();

    if ( ( $form->{op} eq 'userlogin' ) && ( !$user->{is_anon} ) ) {
        my $refer = $form->{returnto} || $ENV{SCRIPT_NAME};
        redirect($refer);
        return;
    }

    # why is this commented out?  -- pudge
    # $form->{mode} = $user->{mode} = "dynamic" if $ENV{SCRIPT_NAME};

    for ( $form->{op} ) {
        my $c;
        upBid( $form->{bid} ), $c++ if /^u$/;
        dnBid( $form->{bid} ), $c++ if /^d$/;
        rmBid( $form->{bid} ), $c++ if /^x$/;
        redirect( $ENV{SCRIPT_NAME} ), return if $c;
    }

    my $section;
    if ( $form->{section} ) {
        $section = $slashdb->getSection( $form->{section} );
    }
    else {
        $section->{section} = 'index';
        $section->{issue}   = 1;
    }

    $section->{artcount} = $user->{maxstories} unless $user->{is_anon};
    $section->{mainsize} = int( $section->{artcount} / 3 );

    my $title = getData( 'head', { section => $section } );
    header( $title, $section->{section} ne 'index' ? $section->{section} : '' );

    my $limit =
        $section->{section} eq 'index'
      ? $user->{maxstories}
      : $section->{artcount};

    my $stories = $slashdb->getStoriesEssentials( $limit,
        ( $form->{section} ne 'index' ) ? $form->{section} : '' );

    my $Stories = displayStories($stories);
    my $StandardBlocks = displayStandardBlocks( $section, $stories );

    slashDisplay(
        'index',
        {
            is_moderator => scalar $slashdb->checkForMetaModerator($user),
            stories      => $Stories,
            boxes        => $StandardBlocks,
        }
    );

    footer();

    writeLog( $form->{section} );
} # handler()

#################################################################
# Should this method be in the DB library?
# absolutely.  we should hide the details there.  but this is in a lot of
# places (modules, index, users); let's come back to it later.  -- pudge
sub saveUserBoxes {
    my (@a)     = @_;
    my $slashdb = getCurrentDB();
    my $user    = getCurrentUser();
    $user->{exboxes} = @a ? sprintf( "'%s'", join "','", @a ) : '';
    $slashdb->setUser( $user->{uid}, { exboxes => $user->{exboxes} } )
      unless $user->{is_anon};
}

#################################################################
sub getUserBoxes {
    my $boxes = getCurrentUser('exboxes');
    $boxes =~ s/'//g;
    return split m/,/, $boxes;
}

#################################################################
sub upBid {
    my ($bid) = @_;
    my @a = getUserBoxes();

    if ( $a[0] eq $bid ) {
        ( $a[0], $a[ @a - 1 ] ) = ( $a[ @a - 1 ], $a[0] );
    }
    else {
        for ( my $x = 1 ; $x < @a ; $x++ ) {
            ( $a[ $x - 1 ], $a[$x] ) = ( $a[$x], $a[ $x - 1 ] )
              if $a[$x] eq $bid;
        }
    }
    saveUserBoxes(@a);
}

#################################################################
sub dnBid {
    my ($bid) = @_;
    my @a = getUserBoxes();
    if ( $a[ @a - 1 ] eq $bid ) {
        ( $a[0], $a[ @a - 1 ] ) = ( $a[ @a - 1 ], $a[0] );
    }
    else {
        for ( my $x = @a - 1 ; $x > -1 ; $x-- ) {
            ( $a[$x], $a[ $x + 1 ] ) = ( $a[ $x + 1 ], $a[$x] )
              if $a[$x] eq $bid;
        }
    }
    saveUserBoxes(@a);
}

#################################################################
sub rmBid {
    my ($bid) = @_;
    my @a = getUserBoxes();
    for ( my $x = @a ; $x >= 0 ; $x-- ) {
        splice @a, $x, 1 if $a[$x] eq $bid;
    }
    saveUserBoxes(@a);
}

#################################################################
sub displayStandardBlocks {
    my ( $section, $older_stories_essentials ) = @_;
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $user      = getCurrentUser();
    my $cache     = getCurrentCache();

    return if $user->{noboxes};

    my ( @boxes, $return, $boxcache );
    my ( $boxBank, $sectionBoxes ) = $slashdb->getPortalsCommon();
    my $getblocks = $section->{section} || 'index';

    # two variants of box cache: one for index with portalmap,
    # the other for any other section, or without portalmap

    if ( $user->{exboxes} && $getblocks eq 'index' ) {
        @boxes = getUserBoxes();
        $boxcache = $cache->{slashboxes}{index_map}{ $user->{light} } ||= {};
    }
    else {
        @boxes = @{ $sectionBoxes->{$getblocks} }
          if ref $sectionBoxes->{$getblocks};
        $boxcache = $cache->{slashboxes}{$getblocks}{ $user->{light} } ||= {};
    }

    for my $bid (@boxes) {
        if ( $bid eq 'mysite' ) {
            $return .= portalbox( $constants->{fancyboxwidth},
                getData('userboxhead'),
                $user->{mylinks} || getData('userboxdefault'), $bid );

        }
        elsif ( $bid =~ /_more$/ && $older_stories_essentials ) {
            $return .= portalbox( $constants->{fancyboxwidth},
                getData('morehead'),
                getOlderStories( $older_stories_essentials, $section ), $bid )
              if @$older_stories_essentials;

        }
        elsif ( $bid eq 'userlogin' && !$user->{is_anon} ) {

            # do nothing!

        }
        elsif ( $bid eq 'userlogin' && $user->{is_anon} ) {
            $return .= $boxcache->{$bid} ||= portalbox(
                $constants->{fancyboxwidth},
                $boxBank->{$bid}{title},
                slashDisplay( 'userlogin', 0, { Return => 1, Nocomm => 1 } ),
                $boxBank->{$bid}{bid},
                $boxBank->{$bid}{url}
            );

        }
        elsif ( $bid eq 'poll' && !$constants->{poll_cache} ) {

            # this is only executed if poll is to be dynamic
            $return .= portalbox(
                $constants->{fancyboxwidth}, $boxBank->{$bid}{title},
                pollbooth( '', 1 ), $boxBank->{$bid}{bid},
                $boxBank->{$bid}{url}
            );

            # this could grab from the cache in the future, perhaps ... ?
        }
        elsif ( $bid eq 'rand' || $bid eq 'srandblock' ) {

            # don't use cached title/bid/url from getPortalsCommon
            my $data = $slashdb->getBlock( $bid, [qw(title block bid url)] );
            $return .= portalbox(
                $constants->{fancyboxwidth},
                @{$data}{qw(title block bid url)}
            );

        }
        else {
            $return .= $boxcache->{$bid} ||= portalbox(
                $constants->{fancyboxwidth}, $boxBank->{$bid}{title},
                $slashdb->getBlock( $bid, 'block' ), $boxBank->{$bid}{bid},
                $boxBank->{$bid}{url}
            );
        }
    }

    return $return;
}

#################################################################
# pass it how many, and what.
sub displayStories {
    my ($stories) = @_;
    my $slashdb   = getCurrentDB();
    my $constants = getCurrentStatic();
    my $form      = getCurrentForm();
    my $user      = getCurrentUser();

    my ( $today, $x ) = ( '', 1 );
    my $cnt = int( $user->{maxstories} / 3 );
    my $return;

    # shift them off, so we do not display them in the Older
    # Stuff block later (simulate the old cursor-based
    # method)
    while ( $_ = shift @{$stories} ) {
        my ( $sid, $thissection, $title, $time, $cc, $d, $hp ) = @{$_};
        my @links;
        my @threshComments = split m/,/, $hp;    # posts in each threshold
        my ( $storytext, $story ) = displayStory($sid);

        $return .= $storytext;

        push @links,
          linkStory(
            {
                'link'  => getData('readmore'),
                sid     => $sid,
                section => $thissection
            }
          );

        if ( $story->{bodytext} || $cc ) {
            push @links,
              linkStory(
                {
                    'link' => length( $story->{bodytext} ) . ' '
                      . getData('bytes'),
                    sid     => $sid,
                    mode    => 'nocomment',
                    section => $thissection
                }
              ) if $story->{bodytext};

            my @cclink;
            my $thresh = $threshComments[ $user->{threshold} + 1 ];

            if ( $cc = $threshComments[0] ) {
                if ( $user->{threshold} > -1 && $cc ne $thresh ) {
                    $cclink[0] = linkStory(
                        {
                            sid       => $sid,
                            threshold => $user->{threshold},
                            'link'    => $thresh,
                            section   => $thissection
                        }
                    );
                }
            }

            $cclink[1] = linkStory(
                {
                    sid       => $sid,
                    threshold => -1,
                    'link'    => $cc || 0,
                    section   => $thissection
                }
            );

            push @cclink, $thresh, ( $cc || 0 );
            push @links, getData( 'comments', { cc => \@cclink } )
              if $cc || $thresh;
        }

        if ( $thissection ne $constants->{defaultsection} && !$form->{section} )
        {
            my ($section) = $slashdb->getSection($thissection);
            push @links,
              getData(
                'seclink',
                {
                    name    => $thissection,
                    section => $section
                },

		  # hacking code 
		  # 'index'
              );
        }

        push @links, getData( 'editstory', { sid => $sid } )
          if $user->{seclev} > 100;

        $return .= slashDisplay(
            'storylink',
            {
                links => \@links,
            },
            { Return => 1 }
        );

        my ($w) = join ' ', ( split m/ /, $time )[ 0 .. 2 ];
        $today ||= $w;
        last if ++$x > $cnt && $today ne $w;
    }

    return $return;
}



1;
